// $Id: mailbox.h,v 1.6 2000/11/14 22:42:41 classic Exp $
//!
// Header file for Mailbox access class
// (c) by Torben Nehmer 2000
// torben@nehmer.net

#include "socket.h"

// STL includes
#include <vector>
#include <algorithm>
#include <iterator>

// libc includes
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>

/**
This class encaspulates an IMAP Mailbox. Each instance of this class will
create a new IMAP connection by itself. You don't have to worry about the
network stuff. All operations of this class are synchronous. Asynchronous
IMAP Operation is currently not supported. [2DO: check for unauthenticated
state during all operations; Add Exceptions]
*/

class Mailbox {
public:
  /** Basic Message Header.
      It doesn't contain Flags or any extended Information like cc or
      X-Headers.
  */
  struct msg_header {
    /// From
    string strFrom;
    /// To
    string strTo;
    /// Date
    string strDate;
    /// Subject
    string strSubject;
  };
    
  /** Data structure with all responses from an EXAMINE or an SELECT
      request. 
  */
  struct mbx_status {
    /// Number of messages
    int iExists;
    /// Number of recent messages
    int iRecent;
    /// Vector of defined FLAGS
    vector<string> vFlags;
    /** Number of first unseen message.
	-1 if server didn't send an UNSEEN response
    */
    int iUnseen;
    /// Vector of defined PERMANTFLAGS
    vector<string> vPermflags;
    /** Unique Identifier Validity value.
	-1 if server didn't send an UIDVALIDITY response
    */
    int iUIDValidity;
    /** Predicted next UID.
	-1 if server didn't send UIDNEXT
    */
    int iUIDNext;
  };


private:
  /// Socket class used for communication with the server.
  Socket m_sockIMAP;

  /// List of the LDAP Server Capabilities
  vector<string> m_vCapabilities;

  /** State of the connection: 0 = non-authenticated, 1 = authenticated,
      2 = selected, 3 = logged out (socket closed)
  */
  int m_iState;

  /// Name of the currently selected mailbox (or "" if none)
  string m_strMailbox;

  /// True, if you have ReadWrite Access to the selected Mailbox.
  bool m_bMailboxReadWrite;
  
  /** The default constructor is private since this object is not default
      constructible. Use the public constructors below instead.
  */
  Mailbox () {};

  /** Establish the socket connection to an IMAP server. It checks all
      capabilities and saves them in m_vCapabilities.
      @param strServer A string containing either a fully qualified host name
      or an IP address in dot notation of the IMAP server.
      @param iPort An integer containing the Port number. This parameter is
      optional. If ommited, it assumes the default IMAP port (143).
      @return True if successful.
  */
  bool connect (string strServer, int iPort);

  /** Read and parse a EXAMINE or SELECT response. returns a mbx_status
      object. The server response is the same for SELECT and EXAMINE. The
      IMAP request has to be done by the caller. parse_select simply
      reads and parses the response.
      @param bSelect true if select, false if examine. 
      @result mbx_status struct with all parsed responses
  */
  mbx_status * parse_select (bool bSelect = true);

  /** Logout from the IMAP Server. System goes into unauthenticated State.
      You have to relogin, an open Mailbox will be closed. This will also
      close the socket connection. The object can not be used any further.
      It is solely designed to be used during the destruction of a Mailbox
      object.
  */
  void logout ();

public:
  /** Standard constructor for the Maibox class. It establishes the socket
      connection to strServer and fills the internal state information with
      all information received from the IMAP server. It leaves the
      connection in an unauthenticated state. Use either the constructor
      with login information or the login function call.
      @param strServer A string containing either a fully qualified host name
      or an IP address in dot notation of the IMAP server.
      @param iPort An integer containing the Port number. This parameter is
      optional. If ommited, it assumes the default IMAP port (143).
  */
  Mailbox (string strServer, int iPort = 143);

  /** This extended constructor establishes the connection and logs you in
      @param strServer A string containing either a fully qualified host name
      or an IP address in dot notation of the IMAP server.
      @param strUser Username to be used for Mailbox Access.
      @param strPassword Password to be used for Mailbox Access
      @param iPort An integer containing the Port number. This parameter is
      optional. If ommited, it assumes the default IMAP port (143).
  */
  Mailbox (string strServer, string strUser, string strPassword,
	   int iPort = 143);

  /** The destructor will log you out of the IMAP server and closes the
      socket connection.
  */
  ~Mailbox ();

  /** This call will log you onto the server specified during construction.
      Be aware the username and password will be transmitted as clear text.
      @param strUser Username to be used for Mailbox Access.
      @param strPassword Password to be used for Mailbox Access
      @return True if successful, false on error (user or password
      incorrect)
  */
  bool login (string strUser, string strPassword);

  /** Select a mailbox with EXAMINE. It returns a structure with all
      responses from the server or a NULL Pointer on error. The EXAMINE
      operation does not change the Mailbox in any way. IMAP gurantees it to
      be a read-only operation. After this call the System is in selected
      state.
      @param strMailbox String with a Mailbox name.
      @return mbx_status struct pointer with all responses from
      the server or a NULL Pointer on error. The calling application has to
      delete this struct.
  */
  mbx_status * examine (string strMailbox);

  /** Select the mailbox. This changes the state of the connection to
      "selected". If a mailbox is already selected it will abort and return
      a NULL Pointer. (This is experimental behavoir as long as the close
      command is not implemented! Normally it should automatically close any
      already selected Mailbox.) It will return a NULL Pointer if no Mailbox
      of the given name exists.
      @param strMailbox Sting with a Mailbox name.
      @return mbx_status struct pointer with all responses from the server or
      a NULL Pointer on error. The calling application has to delete this
      struct.
  */
  mbx_status * select (string strMailbox);

  /** Generic Message search function. Takes a IMAP conform search string and
      returns a list of all message UID's that macht the specified query. Can
      only be done in selected state. If no mailbox is selected, this
      operation will silently fail! (EXPERIMENTAL: No "real" error recovery
      present. It should raise an exception.)
      @param strQuery IMAP compatible query. See RFC 2060. 
      @return STL-vector with the UIDs of all messages, which apply to the
      given query.
  */
  vector<long> search (string strQuery);

  /** Returns a list of UIDs with the RECENT flag set. Use this to get a list
      of all new messages since the last select of the mailbox. Can only be
      done in selected state.
      @return STL-vector with the UIDs of all messages, which have the Flag
      recent set.
  */
  vector<long> search_recent();

  /** Returns a list of UIDs with the UNSEEN flag unset. Use this to
      get a list of all new messages since the last select of the
      mailbox. Can only be done in selected state.
      @return STL-vector with the UIDs of all messages, which have the Flag
      recent set.
  */
  vector<long> search_unseen();

  /** This method fetches the basic header of the message with a given UID.
      @param
  */
  msg_header * getheader (long UID);

  /** Close will send a CLOSE command to the IMAP Server. This will close
      any selected mailbox and silently expunge deleted messages. LOGOUT
      will automatically close any selected Mailbox, as will a new SELECT
      request.
  */
  void close ();

  /** Return state of connection.
      @return The IMAP connection state.
      @see m_iState
  */
  int get_iState();
};
