// $Id: socket.h,v 1.4 2000/11/12 22:45:36 classic Exp $
//
// Header file for socket communication class
// (c) by Torben Nehmer 2000
// torben@nehmer.net

// Unix Socket includes
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

// other includes
#include <unistd.h>
#include <string>


/**
2DO 
- Implement Server Capabilities
- Implement Exception Handling
- Implement buffered I/O
 */

/**
This class encaspulates the Unix Socekt. It can be used with
Stream sockets for IPv4. It uses the STL string type to represent
the sent and received data. This may be a problem with binary
tranfers. This class currently performes unbuffered transfers, aside
from the buffering of the TCP/IP stack. 

The goal is to provide an easy way to access a socket in an object 
oriented style.
 */

class Socket {
private:
  /// Local Socket Descriptor 
  int m_sdSocketDescriptor;
  
  /// Socket Address Structure for local host (needed in a Server situation)
  sockaddr_in m_addrLocal;

  /// Socket Address Structure for remote host 
  sockaddr_in m_addrRemote;



public:
  /** Constant defining the maximum size for unlimited reading.
      Defaults to 64 kB
  */
  static const long MAX_READ_SIZE;
  
  /** Create the socket. It is unbound after the Construction of the object
      has been completed. Use the connect or listen calls. 
  */
  Socket ();

  /// Destuctor. Clean Up Work
  ~Socket (); 
  
  /** Connect to a remote socket. The connect call returns the error
      code of the Unix connect call. If its less than zero, the connect
      failes.

      @param strRemoteAddress Either the IPv4 Address of the remote host or
      the fully qualified host name.
      @param iPort The port number to connect to. It will be automatically
      converted to network byte order.
      @return Result of the connect attempt. 0 if successful.
  */
  int connect (string strRemoteAddress, int iPort);

  /** Bind the socket to a local port. Then activate the listen handler.
      Use the accept call to take over incoming connections. 
      @param iPort Port number to listen on.
      @param iBacklog Length of the listen queue.
      @return Return zero if successful, nonzero on error.
  */
  int listen (int iPort, int iBacklog);

  /** Write data to the socket.
      @param strData STL-String containing the data to be transmitted
      @return Number of bytes written to the remote host
  */
  long write (string strData);

  /** Read data from the socket.
      @param iCount Number of byte to read. If 0 it tries to read as
      much as possible from the input stream, up to MAX_READ_SIZE.
      @return STL-String containing the received data.
  */
  string read (long iCount);

  /** readline reads from the socket until it receives a newline character.
      It does thie byte per bye which isn't very effective.
      @return STL-String containing the received line, including the
      newline character
  */
  string readline ();

  /** This call polls the descriptor for newly available date. It
      returns nonzero if data is available or zero if there was a timeout.
      @param iTimeout Timeout in milliseconds. A negative value will wait
      forever.
      @return Returns zero on timeout, nonzero if data is available
  */
  int pollin (int iTimeout);

  /** The pollhup call checks if the other end of the connection has issued
      a disconnection request. Use this to check if the remote side is still
      there.
      @param iTimeout Timeout in milliseconds. A negative value will wait
      forever.
      @return Returns zero on timeout, nonzero if other side has hung up.
  */
  int pollhup (int iTimeout);
};
